using View_by_Distance.Shared.Models.Properties;

namespace View_by_Distance.Shared.Models.Stateless;

internal abstract class XResult
{

    internal static string GetResultsDateGroupDirectory(IMetadataConfiguration aAConfiguration, string description, string jsonGroup)
    {
        string result = Path.Combine(GetResultsDateGroupDirectory(aAConfiguration, description), jsonGroup);
        if (!Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static string GetResultsDateGroupDirectory(IMetadataConfiguration aAConfiguration, string description)
    {
        string result = Path.Combine(GetResultsGroupDirectory(aAConfiguration, description, create: true), aAConfiguration.DateGroup);
        if (!Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static string GetRelativePath(IMetadataConfiguration aAConfiguration, string path)
    {
        string result = Methods.IPath.GetRelativePath(path, aAConfiguration.RootDirectory.Length);
        return result;
    }

    private static void CheckContent(IMetadataConfiguration aAConfiguration, string dateGroupDirectory, string contentDescription, string result)
    {
        string checkDirectory;
        checkDirectory = Path.Combine(dateGroupDirectory, aAConfiguration.ResultContent, aAConfiguration.ResultAllInOne);
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
        string contentDirectory = new(result.Replace("<>", aAConfiguration.ResultContent));
        if (!Directory.Exists(contentDirectory))
            _ = Directory.CreateDirectory(contentDirectory);
        checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("() - ", contentDescription));
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
    }

    private static void CheckSingleton(IMetadataConfiguration aAConfiguration, string dateGroupDirectory, string singletonDescription, bool converted, string result)
    {
        string checkDirectory;
        checkDirectory = Path.Combine(dateGroupDirectory, aAConfiguration.ResultSingleton, aAConfiguration.ResultAllInOne);
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
        if (!converted)
        {
            string singletonDirectory = new(result.Replace("<>", aAConfiguration.ResultSingleton));
            if (!Directory.Exists(singletonDirectory))
                _ = Directory.CreateDirectory(singletonDirectory);
        }
        checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("{} - ", singletonDescription));
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
    }

    private static void CheckCollection(IMetadataConfiguration aAConfiguration, string dateGroupDirectory, string collectionDescription, bool converted, string result)
    {
        string checkDirectory = Path.Combine(dateGroupDirectory, aAConfiguration.ResultCollection, aAConfiguration.ResultAllInOne);
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
        if (!converted)
        {
            string collectionDirectory = new(result.Replace("<>", aAConfiguration.ResultCollection));
            if (!Directory.Exists(collectionDirectory))
                _ = Directory.CreateDirectory(collectionDirectory);
        }
        checkDirectory = Path.Combine(dateGroupDirectory, string.Concat("[] - ", collectionDescription));
        if (!Directory.Exists(checkDirectory))
            _ = Directory.CreateDirectory(checkDirectory);
    }

    internal static string GetResultsFullGroupDirectory(IMetadataConfiguration aAConfiguration, string description, string outputResolution, bool includeResizeGroup, bool includeModel, bool includePredictorModel)
    {
        string result = GetResultsDateGroupDirectory(aAConfiguration, description);
        if (includeResizeGroup)
            result = Path.Combine(result, outputResolution);
        if (includeModel && includePredictorModel)
        {
            string modelName;
            string predictorModelName;
            if (aAConfiguration.ModelName is null)
                modelName = Model.Hog.ToString();
            else
                modelName = aAConfiguration.ModelName;
            if (aAConfiguration.PredictorModelName is null)
                predictorModelName = PredictorModel.Large.ToString();
            else
                predictorModelName = aAConfiguration.PredictorModelName;
            string dateGroupDirectory = string.Concat(outputResolution.Replace(" ", string.Empty), "-", modelName, "-", predictorModelName, "-", aAConfiguration.NumberOfJitters, "-", aAConfiguration.NumberOfTimesToUpsample);
            result = Path.Combine(result, dateGroupDirectory);
        }
        else if (includeModel)
            throw new Exception();
        else if (includePredictorModel)
            throw new Exception();
        if (!Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static List<string> GetDirectoryInfoCollection(IMetadataConfiguration aAConfiguration, string sourceDirectory, string dateGroupDirectory, string contentDescription, string singletonDescription, string collectionDescription, bool converted)
    {
        List<string> results = [];
        string sourceDirectorySegment = GetRelativePath(aAConfiguration, sourceDirectory);
        string result = string.Concat(Path.Combine(dateGroupDirectory, "<>"), sourceDirectorySegment);
        if (!string.IsNullOrEmpty(contentDescription))
            CheckContent(aAConfiguration, dateGroupDirectory, contentDescription, result);
        if (!string.IsNullOrEmpty(singletonDescription))
            CheckSingleton(aAConfiguration, dateGroupDirectory, singletonDescription, converted, result);
        if (!string.IsNullOrEmpty(collectionDescription))
            CheckCollection(aAConfiguration, dateGroupDirectory, collectionDescription, converted, result);
        results.Add(result);
        return results;
    }

    internal static string GetResultsGroupDirectory(IMetadataConfiguration aAConfiguration, string description, bool create)
    {
        string result = Path.Combine($"{aAConfiguration.RootDirectory}-Results", description.Replace('_', ')'));
        if (create && !Directory.Exists(result))
            _ = Directory.CreateDirectory(result);
        return result;
    }

    internal static List<string> GetDirectoryInfoCollection(IMetadataConfiguration aAConfiguration, string sourceDirectory, string description, string outputResolution, bool includeResizeGroup, bool includeModel, bool includePredictorModel, string contentDescription, string singletonDescription, string collectionDescription)
    {
        List<string> results;
        bool converted = false;
        string dateGroupDirectory = GetResultsFullGroupDirectory(aAConfiguration, description, outputResolution, includeResizeGroup, includeModel, includePredictorModel);
        results = GetDirectoryInfoCollection(aAConfiguration, sourceDirectory, dateGroupDirectory, contentDescription, singletonDescription, collectionDescription, converted);
        return results;
    }

}